/* 
 * vol_mod.c
 *
 * end volume volume
 *
 * Copyright (C) 1998 Hermann Samso
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ncurses.h>
#include "types.h"
#include "delfin.h"
#include "display.h"
#include "wave.h"
#include "module.h"



void VolAddLeft(FLOAT left);
void VolAddRight(FLOAT right);
CHAR* VolBar(FLOAT vol, CHAR* bar);

BOOL loverdrive = TRUE;
BOOL roverdrive = TRUE;
BOOL vol_lock = FALSE;
BOOL vol_lock_lock = FALSE;
	
void VolInit(void) {
	if (trace_on)
		fprintf(stderr, "VolInit()\n");


	ModuleDisplay();
}


void VolExit(void) {
	if (trace_on)
		fprintf(stderr, "VolExit()\n");


}


void VolCtrl(INT input) {
	WAVE* wave = WaveGet();
	FLOAT tmp;
	INT keys[] = {
		'1', '2', '3', '4', '5',	// left volume
		'q', 'w', 'e', 'r', 't',	// right volume
		'a', 's', 'd', 'f', 'g',	// l&r volume
		'6', 'z', 'h', 'n',			// scale WAVE
		'7', 'u', 'j', 'm',
		'c',						// swap l/r volumes
		'v',						// normalize WAVE
		'y',						// toggle left overdrive
		'x',						// toggle right overdrive
		'l'							// lock l&r
	};
	
	if (trace_on)
		fprintf(stderr, "VolCtrl()\n");
		

	// left volume
	if (input == keys[0])
		VolAddLeft(-0.1);
	else if (input == keys[1])
		VolAddLeft(-0.05);
	else if (input == keys[2])
		wave->lvol = 0;
	else if (input == keys[3])
		VolAddLeft(0.05);
	else if (input == keys[4])
		VolAddLeft(0.1);
	// right volume
	else if (input == keys[5])
		VolAddRight(-0.1);
	else if (input == keys[6])
		VolAddRight(-0.05);
	else if (input == keys[7])
		wave->rvol = 0;
	else if (input == keys[8])
		VolAddRight(0.05);
	else if (input == keys[9])
		VolAddRight(0.1);
	else if (input == keys[10]) {
			VolAddLeft(-0.1);
			VolAddRight(-0.1);
	}
	else if (input == keys[11]) {
		VolAddLeft(-32768/100);
		VolAddRight(-32768/100);
	}
	else if (input == keys[12]) {
		wave->lvol = 0;
		wave->rvol = 0;
	}
	else if (input == keys[13]) {
		VolAddLeft(32767/100);
		VolAddRight(32767/100);
	}
	else if (input == keys[14]) {
		VolAddLeft(32767/10);
		VolAddRight(32767/10);
	}
	// scale
	else if (input == keys[15])
		WaveScale(0.99);
	else if (input == keys[16])
		WaveScale(0.95);
	else if (input == keys[17])
		WaveScale(0.90);
	else if (input == keys[18])
		WaveScale(0.5);
	else if (input == keys[19])
		WaveScale(1.01);
	else if (input == keys[20])
		WaveScale(1.05);
	else if (input == keys[21])
		WaveScale(1.1);
	else if (input == keys[22])
		WaveScale(1.5);
	// normalize WAVE
	else if (input == keys[23])
		WaveNormalize();
	// swap l/r volumes
	else if (input == keys[24]) {
		tmp = wave->lvol;
		wave->lvol = wave->rvol;
		wave->rvol = tmp;
	}
	// toggle overdrives
	else if (input == keys[25]) {
		if (loverdrive)
			loverdrive = FALSE;
		else
			loverdrive = TRUE;
	}
	else if (input == keys[26]) {
		if (roverdrive)
			roverdrive = FALSE;
		else
			roverdrive = TRUE;
	}
	else if (input == keys[27]) {
		if (vol_lock)
			vol_lock = FALSE;
		else
			vol_lock = TRUE;
	}
	else
		usleep(200);
					
	ModuleDisplay();
}


void VolExec(void) {
}


void VolDisplay(void) {
	if (trace_on)
		fprintf(stderr, "VolDisplay()\n");


	DisplayClearBottom();
	mvprintw(LINES-2, 0, "[%s]", vol_mod.name);
	mvprintw(LINES-1, 0, "[%d] O-drive [%d]", loverdrive, roverdrive);
	mvprintw(LINES-2, 9, "lock:%d", vol_lock);
	WaveDisplay();
}


//### PRIVATE METHODS ######################################################

void VolAddLeft(FLOAT left) {
	WAVE* wave = WaveGet();

	if (trace_on)
		fprintf(stderr, "VolAddLeft(%f)\n", left);

	if (wave == NULL)
		return;

	wave->lvol = wave->lvol + left;
	if (loverdrive) {
		if (wave->lvol > 1)
			wave->lvol = 1;
		if (wave->lvol < -1)
			wave->lvol = -1;
	}
	if (vol_lock && !vol_lock_lock) {
		vol_lock_lock = TRUE;
		VolAddRight(left);
	}
	vol_lock_lock = FALSE;
}


void VolAddRight(FLOAT right) {
	WAVE* wave = WaveGet();
	
	if (trace_on)
		fprintf(stderr, "VolAddRight(%f)\n", right);
		
	wave->rvol = wave->rvol + right;
	if (roverdrive) {
		if (wave->rvol > 1)
			wave->rvol = 1;
		if (wave->rvol < -1)
			wave->rvol = -1;
	}
	if (vol_lock && !vol_lock_lock) {
		vol_lock_lock = TRUE;
		VolAddLeft(right);
	}
	vol_lock_lock = FALSE;
}


MODULE vol_mod = {
	NULL,			/* next MODULE  in list */
	NULL,			/* previous MODULE in list */
	"volume",
	"v1998.1.22 by Hermann Samso",
	VolInit,
	VolExit,
	VolCtrl,
	VolExec,
	VolDisplay
};
